@@ -90,8 +90,6 @@ def differentials(fluid : Fluid, property : str, parameters):
 
 def outputFromParameters(scalarMethod, property, params):
 
-    
-    
     if (all([type(elem) == scalarVariable for elem in params if not elem is None])):
         ## all inputs are scalars
         out = scalarMethod(property, *params)
@@ -112,7 +110,7 @@ def outputFromParameters(scalarMethod, property, params):
     if ns:
         n = ns[0]
         for i, param in enumerate(params):
-            if isinstance(param, scalarVariable):
+            if not type(param) == arrayVariable:
                 paramVecs[i] = variable([param.value] * n, param.unit, [param.uncert] * n)
         for i in range(n):
             params = [elem[i] for elem in paramVecs]
@@ -153,7 +151,8 @@ def propWaterScalar(property, T,P):
     var = getattr(fluid, property)
     var = variable(var, propertyUnits[property])
     vars, grads = differentials(fluid, property, [T, P])
-    var._addDependents(vars, grads)
+    for v, g in zip(vars, grads):
+        var._addDependent(v,g)
     var._calculateUncertanty()
     
     ## return the variable
@@ -203,7 +202,8 @@ def propMEGScalar(property, T,P,C):
     var = getattr(fluid, property)
     var = variable(var, propertyUnits[property])
     vars, grads = differentialsBrine(fluid, FluidsList.MEG, property, C, [T, P])
-    var._addDependents(vars, grads)
+    for v, g in zip(vars, grads):
+        var._addDependent(v,g)
     var._calculateUncertanty()
     
     ## convert the arguments back in to the original units
@@ -276,8 +276,9 @@ def propHumidAirScalar(property, T,Rh,H = None, P = None):
     ## create a variable from the fluid
     var = getattr(fluid, property)
     var = variable(var, propertyUnits[property])
-    Vars, grads = differentials(fluid, property, vars)
-    var._addDependents(Vars, grads)
+    vars, grads = differentials(fluid, property, vars)
+    for v, g in zip(vars, grads):
+        var._addDependent(v,g)
     var._calculateUncertanty()
     
     return var 
@@ -301,7 +302,9 @@ knownFluids = {
 }
 
 
-if __name__ == '__main__':
-    rho = prop('density', 'water', P = variable(1,'bar'), T = variable(30,'C'))
-    print(rho)
+if __name__ == "__main__":
+    
+    T = variable([20,30], 'C', [1,1])
+    P = variable(1, 'bar', 0.01)
+    C = variable(50,'%', 2)
     
\ No newline at end of file
