@@ -17,12 +17,22 @@ class test(unittest.TestCase):
             x = variable(x, 'm')
             y = variable(y, 'C', uncert=[1] * i)
             F = pol_fit(x, y, deg=0)
-            Fa = F.popt[0]
+            Fa = F.coefficients[0]
             self.assertAlmostEqual(Fa.value, 10)
             self.assertEqual(str(Fa.unit), 'C')
             self.assertAlmostEqual(Fa.uncert, 1 / np.sqrt(i))
             self.assertAlmostEqual(F.r_squared, 1)
 
+        ## fit fromtable 6.1 in "Measurements and their uncertanties" by Ifan G. Hughes
+        frequency = variable([10,20,30,40,50,60,70,80,90,100,110], 'Hz')
+        voltage = variable([16, 45, 64, 75,70,115, 142, 167, 183, 160, 221], 'mV', [5,5,5,5,30,5,5,5,5,30,5])
+        fit = lin_fit(frequency, voltage)
+        
+        a,b = fit.coefficients
+        self.assertEqual(str(a), '2.03 +/- 0.05 [mV/Hz]')
+        self.assertEqual(str(b), '-1 +/- 3 [mV]')
+        
+
     def testLinFit(self):
         a = 2
         b = 10
@@ -34,8 +44,8 @@ class test(unittest.TestCase):
         y = variable(y, 'C')
 
         F = lin_fit(x, y)
-        Fa = F.popt[0]
-        Fb = F.popt[1]
+        Fa = F.coefficients[0]
+        Fb = F.coefficients[1]
 
         self.assertAlmostEqual(Fa.value, 2)
         self.assertEqual(str(Fa.unit), 'DELTAC/m')
@@ -60,9 +70,9 @@ class test(unittest.TestCase):
         y = variable(y, 'C')
 
         F = pol_fit(x, y)
-        Fa = F.popt[0]
-        Fb = F.popt[1]
-        Fc = F.popt[2]
+        Fa = F.coefficients[0]
+        Fb = F.coefficients[1]
+        Fc = F.coefficients[2]
 
         self.assertAlmostEqual(Fa.value, 2)
         self.assertEqual(str(Fa.unit), 'DELTAC/m2')
@@ -92,10 +102,10 @@ class test(unittest.TestCase):
         y = variable(y, 'C')
 
         F = pol_fit(x, y, deg=3)
-        Fa = F.popt[0]
-        Fb = F.popt[1]
-        Fc = F.popt[2]
-        Fd = F.popt[3]
+        Fa = F.coefficients[0]
+        Fb = F.coefficients[1]
+        Fc = F.coefficients[2]
+        Fd = F.coefficients[3]
 
         self.assertAlmostEqual(Fa.value, 2)
         self.assertEqual(str(Fa.unit), 'DELTAC/m3')
@@ -116,5 +126,10 @@ class test(unittest.TestCase):
         self.assertAlmostEqual(F.r_squared, 1)
 
 
+## TODO test exp_fit
+## TODO test pow_fit
+## TODO test logistic_fit    
+
+
 if __name__ == '__main__':
     unittest.main()
