@@ -79,7 +79,17 @@ class test(unittest.TestCase):
         b = unit('K-m/bar-L')
         _,cUnit,_,_,_ = a + b
         self.assertTrue(unit._assertEqualStatic(cUnit, 'DELTAK-m/bar-L'))
-
+        
+        a = unit('J/kg-DELTAK')
+        b = unit('J/kg-DELTAK')
+        _,cUnit,_,_,_ = a + b
+        self.assertTrue(unit._assertEqualStatic(cUnit, 'J/kg-DELTAK'))
+        
+        a = unit('J/g-DELTAK')
+        b = unit('J/kg-DELTAK')
+        _,cUnit,_,_,_ = a + b
+        self.assertTrue(unit._assertEqualStatic(cUnit, 'J/g-DELTAK'))
+        
     def testSub(self):
         a = unit('L/min')
         b = unit('kg-m/L')
@@ -87,12 +97,21 @@ class test(unittest.TestCase):
         with self.assertRaises(Exception) as context:
             a-b
         self.assertTrue('You tried to subtract a variable in [kg-m/L] from a variable in [L/min], but the units do not have the same SI base unit' in str(context.exception))
-
         
         a = unit('L/min')
         b = unit('L/min')
         _,cUnit,_,_,_ = a - b
         self.assertEqual(str(cUnit), 'L/min')
+        
+        a = unit('J/kg-DELTAK')
+        b = unit('J/kg-DELTAK')
+        _,cUnit,_,_,_ = a - b
+        self.assertTrue(unit._assertEqualStatic(cUnit, 'J/kg-DELTAK'))
+        
+        a = unit('J/g-DELTAK')
+        b = unit('J/kg-DELTAK')
+        _,cUnit,_,_,_ = a - b
+        self.assertTrue(unit._assertEqualStatic(cUnit, 'J/g-DELTAK'))
 
     def testConvert(self):
         a = unit('L/min')
@@ -152,6 +171,9 @@ class test(unittest.TestCase):
 
     def testInput(self):
 
+        a = unit(None)
+        self.assertEqual(str(a), '1')
+        
         a = unit('m / s')
         self.assertEqual(str(a), 'm/s')
 
@@ -163,6 +185,26 @@ class test(unittest.TestCase):
             a = unit('m/s/bar')
         self.assertEqual('A unit can only have a single slash (/)', str(context.exception))
 
+        self.assertEqual(str(unit(' ')), '1')
+        self.assertEqual(str(unit('-')), '1')
+        self.assertEqual(str(unit('')), '1')
+        self.assertEqual(str(unit('--')), '1')
+        self.assertEqual(str(unit('- -')), '1')
+        
+        self.assertEqual(str(unit('()')), '1')
+        self.assertEqual(str(unit('( )')), '1')
+        self.assertEqual(str(unit('(  )')), '1')  
+        self.assertEqual(str(unit('(-)')), '1')
+        self.assertEqual(str(unit('(--)')), '1')
+        self.assertEqual(str(unit('(- -)')), '1')
+        self.assertEqual(str(unit('( -)')), '1')
+        self.assertEqual(str(unit('( - (- ))')), '1')
+        
+        
+        self.assertEqual(str(unit('(m/s2)2/Hz')), 'm2/s4-Hz')
+        self.assertEqual(str(unit('(m1/s2)2/Hz')), 'm2/s4-Hz')
+        self.assertEqual(str(unit('(m1/s2)1/Hz')), 'm/s2-Hz')
+
     def testAddNewUnit(self):
         addNewUnit('gnA', 9.81, 'm/s2')
         converter = unit('gnA').getConverter('m/s2')
@@ -198,9 +240,8 @@ class test(unittest.TestCase):
         converter = unit('Ra').getConverter('Rø')
         self.assertAlmostEqual(converter.convert(83.1), -111.66625)
         
-        
-        
-        
+
+    
 
 if __name__ == '__main__':
     unittest.main()
