@@ -150,6 +150,21 @@ class test(unittest.TestCase):
             A_vec + B_vec
         self.assertTrue('You tried to add a variable in [L/min] to a variable in [m], but the units do not have the same SI base unit' in str(context.exception))
 
+        a = variable(10, 'L', 1.2)
+        b = variable(100, 'mL', 3.9)
+        c = a + b
+        self.assertEqual(c.value, 10.1)
+        self.assertEqual(c.unit, 'L')
+        self.assertEqual(c.uncert, np.sqrt((1 * 1.2)**2 + (1 * 3.9/1000)**2))
+        
+        
+        a = variable(10, 'C', 1.2)
+        b = variable(100, 'muC', 3.9)
+        c = a + b
+        self.assertEqual(c.value, 10 + 100 * 1e-6)
+        self.assertEqual(c.unit, 'C')
+        self.assertEqual(c.uncert, np.sqrt((1 * 1.2)**2 + (1 * 3.9*1e-6)**2))
+
     def test_sub_with_different_units(self):
         A = variable(12.3, 'm3/s', uncert=2.6)
         B = variable(745.1, 'L/min', uncert=53.9)
@@ -193,6 +208,20 @@ class test(unittest.TestCase):
             A_vec - B_vec
         self.assertTrue('You tried to subtract a variable in [m] from a variable in [L/min], but the units do not have the same SI base unit' in str(context.exception))
 
+        a = variable(10, 'L', 1.2)
+        b = variable(100, 'mL', 3.9)
+        c = a - b
+        self.assertEqual(c.value, 10 - 100 / 1000)
+        self.assertEqual(c.unit, 'L')
+        self.assertEqual(c.uncert, np.sqrt((1 * 1.2)**2 + (1 * 3.9/1000)**2))
+        
+        a = variable(10, 'C', 1.2)
+        b = variable(100, 'muC', 3.9)
+        c = a - b
+        self.assertAlmostEqual(c.value, 10 - 100 * 1e-6)
+        self.assertEqual(c.unit, 'DELTAC')
+        self.assertEqual(c.uncert, np.sqrt((1 * 1.2)**2 + (1 * 3.9*1e-6)**2))
+
     def test_multiply(self):
         A = variable(12.3, 'L/min', uncert=2.6)
         B = variable(745.1, 'm', uncert=53.9)
@@ -208,7 +237,7 @@ class test(unittest.TestCase):
         C_vec = A_vec * B_vec
         np.testing.assert_array_equal(C_vec.value, np.array([12.3 * 745.1, 54.3 * 496.13, 91.3 * 120.54]))
         self.assertTrue(C._unitObject._assertEqual('L-m/min'))
-        np.testing.assert_array_equal(
+        np.testing.assert_array_almost_equal(
             C_vec.uncert,
             np.array([
                 np.sqrt((745.1 * 2.6)**2 + (12.3 * 53.9)**2),
@@ -253,7 +282,7 @@ class test(unittest.TestCase):
         C_vec = A_vec / B_vec
         np.testing.assert_array_equal(C_vec.value, np.array([12.3 / 745.1, 54.3 / 496.13, 91.3 / 120.54]))
         self.assertTrue(C_vec._unitObject._assertEqual('L/min-m'))
-        np.testing.assert_array_equal(
+        np.testing.assert_array_almost_equal(
             C_vec.uncert,
             np.array([
                 np.sqrt((1 / 745.1 * 2.6)**2 + (12.3 / (745.1)**2 * 53.9)**2),
@@ -451,31 +480,36 @@ class test(unittest.TestCase):
         A_vec = variable([12.3, 54.3, 91.3], 'L/min', uncert=[2.6, 5.4, 10.56])
 
         with self.assertRaises(Exception) as context:
-            a = A[0]
+            A[0]
         self.assertTrue("'scalarVariable' object is not subscriptable" in str(context.exception))
+
+        with self.assertRaises(Exception) as context:
+            A[1]
+        self.assertTrue("'scalarVariable' object is not subscriptable" in str(context.exception))
+
+        with self.assertRaises(Exception) as context:
+            A_vec[23]
+        self.assertTrue('list index out of range' in str(context.exception))
         
-        a_vec = A_vec[0, 1]
-        np.testing.assert_equal(a_vec.value, [12.3, 54.3])
+        a_vec = A_vec[0]
+        self.assertEqual(a_vec.value, 12.3)
         self.assertEqual(a_vec.unit, 'L/min')
-        np.testing.assert_equal(a_vec.uncert, [2.6, 5.4])
-
-        a_vec = A_vec[0, 2]
-        np.testing.assert_equal(a_vec.value, [12.3, 91.3])
+        self.assertEqual(a_vec.uncert, 2.6)
+        
+        a_vec = A_vec[0:2]
+        np.testing.assert_array_equal(a_vec.value, [12.3, 54.3])
         self.assertEqual(a_vec.unit, 'L/min')
-        np.testing.assert_equal(a_vec.uncert, [2.6, 10.56])
-
-        a_vec = A_vec[2, 0]
-        np.testing.assert_equal(a_vec.value, [91.3, 12.3])
+        np.testing.assert_array_equal(a_vec.uncert, [2.6, 5.4])
+        
+        a_vec = A_vec[1:3]
+        np.testing.assert_array_equal(a_vec.value, [54.3, 91.3])
         self.assertEqual(a_vec.unit, 'L/min')
-        np.testing.assert_equal(a_vec.uncert, [10.56, 2.6])
-
-        with self.assertRaises(Exception) as context:
-            a = A[1]
-        self.assertTrue("'scalarVariable' object is not subscriptable" in str(context.exception))
-
-        with self.assertRaises(Exception) as context:
-            a = A_vec[23]
-        self.assertTrue('Index out of bounds' in str(context.exception))
+        np.testing.assert_array_equal(a_vec.uncert, [5.4, 10.56])
+        
+        a_vec = A_vec[[0,2]]
+        np.testing.assert_array_equal(a_vec.value, [12.3, 91.3])
+        self.assertEqual(a_vec.unit, 'L/min')
+        np.testing.assert_array_equal(a_vec.uncert, [2.6, 10.56])
 
     def testAddEqual(self):
         A = variable(12.3, 'L/min', uncert=2.6)
@@ -861,7 +895,6 @@ class test(unittest.TestCase):
         v = np.sqrt(2 * dP / rho)
         self.assertEqual(v.value, 8.9442719099991585541)
 
-
     def testLargerUncertThenValue(self):
 
         A = variable(0.003, 'L/min', 0.2)
@@ -883,7 +916,12 @@ class test(unittest.TestCase):
         self.assertEqual(str(A), '0.05 +/- 0.07 [L/min]')
 
         A = variable(0.0543, 'L/min', 0.7)
-        self.assertEqual(str(A), '0.0 +/- 0.7 [L/min]')
+        self.assertEqual(str(A), '0.1 +/- 0.7 [L/min]')
+        
+        A = variable(0.9, 'L/min', 3)
+        self.assertEqual(str(A), '1 +/- 3 [L/min]')
+        
+        
 
     def testUnitless(self):
         with self.assertRaises(Exception) as context:
@@ -941,7 +979,6 @@ class test(unittest.TestCase):
         np.testing.assert_equal(d_vec.value, [2**0.34, 2**0.64, 2**0.87])
         self.assertEqual(d_vec.unit, '1')
         self.assertEqual(d_vec.uncert[0], np.sqrt((2**0.34 * np.log(2) * 0.01)**2 + (0.34 * 2**(0.34 - 1) * 0)**2))
-
         
     def testPrettyPrint(self):
         a = variable(12.3, 'm')
@@ -1091,7 +1128,7 @@ class test(unittest.TestCase):
         unc = 2
         self.assertEqual(c.value, val * np.sin(np.pi / 180 * val))
         self.assertEqual(c.unit, 'deg')
-        self.assertEqual(c.uncert, np.sqrt((unc * (np.sin(np.pi / 180 * val) + (np.pi / 180 * val) * np.cos(np.pi / 180 * val)))**2))
+        self.assertAlmostEqual(c.uncert, np.sqrt((unc * (np.sin(np.pi / 180 * val) + (np.pi / 180 * val) * np.cos(np.pi / 180 * val)))**2))
 
         a = variable(23, 'deg', 2)
         a.convert('rad')
@@ -1121,7 +1158,7 @@ class test(unittest.TestCase):
         unc = 2
         self.assertEqual(c.value, val * np.sin(np.pi / 180 * val))
         self.assertEqual(c.unit, 'deg')
-        self.assertEqual(c.uncert, np.sqrt((unc * (np.sin(np.pi / 180 * val) + (np.pi / 180 * val) * np.cos(np.pi / 180 * val)))**2))
+        self.assertAlmostEqual(c.uncert, np.sqrt((unc * (np.sin(np.pi / 180 * val) + (np.pi / 180 * val) * np.cos(np.pi / 180 * val)))**2))
 
         a = variable(200, 'L/min', 1.5)
         b = a**2
@@ -1192,7 +1229,7 @@ class test(unittest.TestCase):
         a = variable(2.3, 'L/min', 0.0237)
         b = variable(0, 'L/min')
         for i in range(15):
-            b += 1 / variable(np.math.factorial(i)) * a
+            b += 1 / np.math.factorial(i) * a
         self.assertAlmostEqual(b.value, np.e * 2.3)
         self.assertEqual(b.unit, 'L/min')
         self.assertAlmostEqual(b.uncert, np.sqrt((np.e * 0.0237)**2))
@@ -1200,22 +1237,44 @@ class test(unittest.TestCase):
     def testCovariance(self):
         a = variable(123, 'L/min', 9.7)
         b = variable(93, 'Pa', 1.2)
-        a._addCovariance(b, [23])
-        b._addCovariance(a, [23])
+        a.addCovariance(b, 23, 'L-Pa/min')
         c = a * b
         self.assertEqual(c.value, 123 * 93)
         self.assertTrue(c._unitObject._assertEqual('L-Pa/min'))
-        self.assertEqual(c.uncert, np.sqrt((123 * 1.2)**2 + (93 * 9.7)**2 + 2 * 93 * 123 * 23))
+        self.assertAlmostEqual(c.uncert, np.sqrt((123 * 1.2)**2 + (93 * 9.7)**2 + 2 * 93 * 123 * 23))
 
         a = variable(123, 'L/min', 9.7)
         b = variable(93, 'Pa', 1.2)
-        a._addCovariance(b, [23])
-        b._addCovariance(a, [23])
+        a.addCovariance(b, 23, 'L-Pa/min')
+        a.convert('m3/s')
+        c = a * b
+        self.assertEqual(c.value, 123 * 93 / 1000 / 60)
+        self.assertTrue(c._unitObject._assertEqual('m3-Pa/s'))
+        self.assertEqual(c.uncert, np.sqrt((123 / 1000 / 60 * 1.2)**2 + (93 * 9.7 / 1000 / 60)**2 + 2 * 93 * 123 / 1000 / 60 * 23 / 1000 / 60))
+        
+        a = variable(123, 'L/min', 9.7)
+        b = variable(93, 'Pa', 1.2)
+        a.addCovariance(b, 23, 'm3-Pa/s')
         a.convert('m3/s')
         c = a * b
         self.assertEqual(c.value, 123 * 93 / 1000 / 60)
         self.assertTrue(c._unitObject._assertEqual('m3-Pa/s'))
         self.assertEqual(c.uncert, np.sqrt((123 / 1000 / 60 * 1.2)**2 + (93 * 9.7 / 1000 / 60)**2 + 2 * 93 * 123 / 1000 / 60 * 23))
+        
+        a = variable([1, 2, 3], 'L/min', [0.1, 0.2 ,0.3])
+        b = variable([93, 97, 102], 'Pa', [1.2, 2.4, 4.7])
+        a.addCovariance(b, [2, 3, 4], 'L-Pa/min')
+        c = a * b
+        np.testing.assert_equal(c.value, [1*93, 2*97, 3*102])
+        self.assertTrue(c._unitObject._assertEqual('L-Pa/min'))
+        dcda = np.array([93, 97, 102], dtype = float)
+        dcdb = np.array([1, 2, 3], dtype = float)
+        ua = np.array([0.1, 0.2, 0.3], dtype = float)
+        ub = np.array([1.2, 2.4, 4.7], dtype = float)
+        uab = np.array([2, 3, 4], dtype = float)
+
+        uc = np.sqrt((dcda * ua)**2 + (dcdb * ub)**2 + 2 * dcda * dcdb * uab)
+        np.testing.assert_equal(c.uncert, uc)
 
     def testConvert(self):
         a = variable(1, 'km')
@@ -1335,7 +1394,71 @@ class test(unittest.TestCase):
         with self.assertRaises(Exception) as context:
             A[0] = B
         self.assertTrue("'scalarVariable' object does not support item assignment" in str(context.exception))
-           
+          
+        a = variable([1,2,3], 'm', [0.1, 0.2, 0.3])
+        b = a**2
+        a[1] = variable(5,'m',0.5)
+        c = b * a
+        
+        a0 = variable(1, 'm', 0.1)
+        a1 = variable(2, 'm', 0.2)
+        a2 = variable(3, 'm', 0.3)
+        b0 = a0**2
+        b1 = a1**2
+        b2 = a2**2
+        a1 = variable(5,'m', 0.5)
+        c0 = b0 * a0
+        c1 = b1 * a1
+        c2 = b2 * a2
+        
+        self.assertEqual(c[0].value, c0.value)
+        self.assertEqual(c[1].value, c1.value)
+        self.assertEqual(c[2].value, c2.value)
+        self.assertEqual(c.unit, c0.unit)
+        self.assertEqual(c[0].uncert, c0.uncert)
+        self.assertEqual(c[1].uncert, c1.uncert)
+        self.assertEqual(c[2].uncert, c2.uncert)
+        
+        
+        A = variable([1, 2, 3], 'L/min', [0.1, 0.2 ,0.3])
+        B = variable([93, 97, 102], 'Pa', [1.2, 2.4, 4.7])
+        A.addCovariance(B, [2, 3, 4], 'L-Pa/min')
+        C = A * B
+        A[1] = variable(2.5, 'L/min', 0.25)
+        C *= A
+                
+        a0 = variable(1, 'L/min', 0.1)
+        b0 = variable(93, 'Pa', 1.2)
+        a0.addCovariance(b0, 2, 'L-Pa/min')
+        c0 = a0 * b0
+        c0 *= a0
+        
+        a1 = variable(2, 'L/min', 0.2)
+        b1 = variable(97, 'Pa', 2.4)
+        a1.addCovariance(b1, 3, 'L-Pa/min')
+        c1 = a1 * b1
+        a11 = variable(2.5, 'L/min', 0.25)
+        c1 *= a11
+        
+        a2 = variable(3, 'L/min', 0.3)
+        b2 = variable(102, 'Pa', 4.7)
+        a2.addCovariance(b2, 4, 'L-Pa/min')
+        c2 = a2 * b2
+        c2 *= a2
+        
+        np.testing.assert_equal(C[0].value, c0.value)
+        self.assertTrue(C._unitObject._assertEqual(c0.unit))
+        np.testing.assert_equal(C[0].uncert, c0.uncert)
+        
+        np.testing.assert_equal(C[1].value, c1.value)
+        self.assertTrue(C._unitObject._assertEqual(c0.unit))
+        np.testing.assert_equal(C[1].uncert, c1.uncert)
+        
+        np.testing.assert_equal(C[2].value, c2.value)
+        self.assertTrue(C._unitObject._assertEqual(c0.unit))
+        np.testing.assert_equal(C[2].uncert, c2.uncert)
+
+              
     def testAppend(self):
         A = variable(12.3, 'L/min', uncert=2.6)
         B = variable(45, 'Pa', 1.2)
@@ -1359,6 +1482,33 @@ class test(unittest.TestCase):
             A.append(B)
         self.assertTrue("'scalarVariable' object has no attribute 'append'" in str(context.exception))
         
+        a = variable([1, 2, 3], 'm')
+        b = variable([4, 5, 6], 'm')
+        c = variable([10, 11, 12], 'Pa')
+        d = variable([13, 14, 15], 'Pa')
+        b.addCovariance(d, [0.1, 0.2, 0.3], 'm-Pa')
+        a.append(b)
+        c.append(d) 
+        d = a * c
+
+        np.testing.assert_equal(d.value, np.array([1,2,3,4,5,6]) * np.array([10,11,12,13,14,15]))
+        self.assertTrue(d._unitObject._assertEqualStatic(d.unit, 'm-Pa'))        
+        np.testing.assert_equal(d.uncert, np.sqrt(2 * np.array([1,2,3,4,5,6]) * np.array([10,11,12,13,14,15]) * np.array([0,0,0,0.1,0.2,0.3])))
+        
+        
+        a = variable([1, 2, 3], 'm')
+        b = variable([4, 5, 6], 'm')
+        c = variable([10, 11, 12], 'Pa')
+        d = variable([13, 14, 15], 'Pa')
+        b.addCovariance(c, [0.1, 0.2, 0.3], 'm-Pa')
+        a.append(b)
+        c.append(d) 
+        d = a * c
+        np.testing.assert_equal(d.value, np.array([1,2,3,4,5,6]) * np.array([10,11,12,13,14,15]))
+        self.assertTrue(d._unitObject._assertEqualStatic(d.unit, 'm-Pa'))        
+        np.testing.assert_equal(d.uncert, np.array([0,0,0,0,0,0]))
+        
+        
     def testAddBel(self):
         a = variable(11,'dB', 0.1)
         b = variable(19,'dB',1.2)
