@@ -1,4 +1,4 @@
-from copy import deepcopy
+from copy import copy
 import numpy as np
 try:
     from unit import unit
@@ -94,26 +94,24 @@ class logarithmicVariables:
         return c
    
 class scalarVariable():
-    def __init__(self, value, unitStr, uncert, nDigits) -> None:
+    def __init__(self, value, unitObject : str | unit, uncert, nDigits) -> None:
         
         self._value = value
         self._uncert = uncert
 
         # create a unit object
-        self._unitObject = unitStr if isinstance(unitStr, unit) else unit(unitStr)
+        self._unitObject = unitObject if isinstance(unitObject, unit) else unit(unitObject)
 
         # number of digits to show
         self.nDigits = nDigits
 
-        # value and unit in SI. This is used when determining the gradient in the uncertanty expression
-        self._getConverterToSI()
-
         # uncertanty
         self.dependsOn = {}
         self.covariance = {}
 
-    def _getConverterToSI(self):
-        self._converterToSI = self._unitObject._converterToSI
+    @property
+    def _converterToSI(self):
+        return self._unitObject._converterToSI
 
     @property
     def value(self):
@@ -132,9 +130,6 @@ class scalarVariable():
         self._value = converter.convert(self._value, useOffset=not self._unitObject.isCombinationUnit())
         self._uncert = converter.convert(self._uncert, useOffset=False)
         self._unitObject.convert(newUnit)
-
-        # update the converter to SI
-        self._getConverterToSI()
  
     def printUncertanty(self, value, uncert):
         # function to print number
@@ -162,9 +157,9 @@ class scalarVariable():
             else:
                 value = int(np.around(value, - nDecimals + 1))
         else:
-            value = '0'
-            if digitsUncert > 0:
-                value += '.' + ''.join(['0'] * digitsUncert)
+            value = str(np.around(value, digitsUncert))
+            if '.' in value and digitsUncert <= 0:
+                value = value.split('.')[0]
 
         return value, uncert
 
@@ -203,64 +198,87 @@ class scalarVariable():
         else:
                 return rf'{value} {pm} {uncert}{space}{unitStr}'
 
-    def _addDependents(self, vars, grads):
-        # copy the gradients
-        # this prevents a pointer error as the gradients often are set to the value of a variable
-        # when the gradients are scaled this causes problems
-        grads = [deepcopy(elem) for elem in grads]
-
-        # loop over the variables and their gradients
-        for var, grad in zip(vars, grads):
-            # scale the gradient to SI units. This is necessary if one of the variables are converted after the dependency has been noted
-            scale = self._converterToSI.convert(1, useOffset=False) / var._converterToSI.convert(1, useOffset=False)
-            grad *= scale
-
-            # check if the variable depends on other variables
-            if var.dependsOn:
-
-                # loop over the dependencies of the variables and add them to the dependencies of self.
-                # this ensures that the product rule is used
-                for key, item in var.dependsOn.items():
-                    if key in self.dependsOn:
-                        self.dependsOn[key] += item * grad
-                    else:
-                        self.dependsOn[key] = item * grad
-            else:
-
-                # the variable did not have any dependecies. Therefore the the varaible is added to the dependencies of self
-                if var in self.dependsOn:
-                    self.dependsOn[var] += grad
+    def _addDependent(self, var, grad):
+        # scale the gradient to SI units. This is necessary if one of the variables are converted after the dependency has been noted
+        grad *= self._converterToSI.convert(1, useOffset=False) / var._converterToSI.convert(1, useOffset=False)
+        
+        if var.dependsOn:
+            # the variable depends on other variables
+            # loop over the dependencies of the variables and add them to the dependencies of self.
+            # this ensures that the product rule is used
+            for vvar, dependency in var.dependsOn.items():
+                if not vvar in self.dependsOn:
+                    unc = vvar._converterToSI.convert(vvar.uncert, useOffset = False)
+                    self.dependsOn[vvar] = [unc, grad * dependency[1]]
                 else:
-                    self.dependsOn[var] = grad
+                    self.dependsOn[vvar][1] += grad * dependency[1]
+        else:    
+            ## the variable does not depend on other variables
+            ## add the variable to the dependencies of self
+            if not var in self.dependsOn:
+                unc = var._converterToSI.convert(var.uncert, useOffset = False)
+                self.dependsOn[var] = [unc, grad]
+            else:
+                self.dependsOn[var][1] += grad
 
-    def _addCovariance(self, var, covariance):
-        self.covariance[var] = covariance
+    def __iter__(self):
+        self.n = 0
+        return self
 
+    def __next__(self):
+        if self.n == 0:
+            self.n += 1
+            return self
+        raise StopIteration           
+   
+    def addCovariance(self, var, covariance: float, unitStr: str):
+        try:
+            float(covariance)
+        except TypeError:
+            raise ValueError(f'You tried to set the covariance between {self} and {var} with a non scalar value')
+        
+        covUnit = unit(unitStr)
+        selfVarUnit = unit(self._unitObject * var._unitObject)
+        if not unit._assertEqualStatic(covUnit._SIBaseUnit, selfVarUnit._SIBaseUnit):
+            raise ValueError(f'The covariance of {covariance} [{unitStr}] does not match the units of {self} and {var}')
+        
+        covariance = covUnit._converterToSI.convert(covariance, useOffset=False)
+        
+        self.covariance[var] = covariance        
+        var.covariance[self] = covariance
+        
     def _calculateUncertanty(self):
-
+ 
         # variance from each measurement
         variance = 0
-        selfScaleToSI = self._converterToSI.convert(1, useOffset=False)
-        for var, grad in self.dependsOn.items():
-            # the gradient is scaled with the inverse of the conversion of the unit to SI units.
-            # This is necessary if the variable "var" has been converted after the dependency has been noted
-            scale = var._converterToSI.convert(1, useOffset=False) / selfScaleToSI           
-            variance += (grad * scale * var.uncert)**2
 
+        # loop over each different variable object in the dependency dict
+        for dependency in self.dependsOn.values():
+            
+            ## loop over each "instance" of the variable
+            ## an instance occurs when the variable has been changed using methods as __setitem__
+            ## this affectively makes the variable a "new" variable in the eyes of other variables
+    
+            ## add the variance constribution to the variance
+            unc, grad = dependency[0], dependency[1]
+            variance += (unc * grad) ** 2
+        
+        ## scale the variance back in to the currenct unit
+        ## the scaling is raised to a power of 2, as the above line should be
+        ## variance += (scale * unc * grad) ** 2
+        scale = 1 / self._converterToSI.convert(1, useOffset=False)
+        variance *= scale ** 2
+        
         # variance from the corralation between measurements
-        n = len(self.dependsOn.keys())
-        for i in range(n):
-            var_i = list(self.dependsOn.keys())[i]
-            for j in range(i + 1, n):
-                var_j = list(self.dependsOn.keys())[j]
-                if var_j in var_i.covariance.keys():
-                    if not var_i in var_j.covariance.keys():
-                        raise ValueError(
-                            f'The variable {var_i} is correlated with the varaible {var_j}. However the variable {var_j} not not correlated with the variable {var_i}. Something is wrong.')
-                    scale_i = var_i._converterToSI.convert(1, useOffset=False) / selfScaleToSI
-                    scale_j = var_j._converterToSI.convert(1, useOffset=False) / selfScaleToSI
-                    varianceContribution = 2 * scale_i * self.dependsOn[var_i] * scale_j * self.dependsOn[var_j] * var_i.covariance[var_j][0]
-                    variance += varianceContribution
+        ## covariance = list(vari, vari.currentValue, vari.currentUncert, varj, varj.currentValue, varj.currenctUncert, cov)
+        ## from the above the gradients can be found from self.dependsOn
+        
+        for var1 in self.dependsOn.keys():
+            for var2 in var1.covariance.keys():
+                if var2 in self.dependsOn:
+                    grad1 = self.dependsOn[var1][1]
+                    grad2 = self.dependsOn[var2][1]
+                    variance += scale**2 * grad1 * grad2 * var1.covariance[var2]
 
         self._uncert = np.sqrt(variance)
         
@@ -276,8 +294,8 @@ class scalarVariable():
             return logarithmicVariables.__add__(self, other)
 
         # convert self and other to the SI unit system
-        selfUnit = deepcopy(self.unit)
-        otherUnit = deepcopy(other.unit)
+        selfUnit = copy(self.unit)
+        otherUnit = copy(other.unit)
         
         if scaleToSI:
             self.convert(self._unitObject._SIBaseUnit)
@@ -292,13 +310,12 @@ class scalarVariable():
         
 
         # determine the value and gradients
-        val = self._value + other._value
-        grad = [1, 1]
-        vars = [self, other]
+        val = self.value + other.value
         
         # create the new variable
         var = variable(val, outputUnit)
-        var._addDependents(vars, grad)
+        var._addDependent(self, 1)
+        var._addDependent(other, 1)
         var._calculateUncertanty()
 
         # convert all units back to their original units
@@ -330,8 +347,8 @@ class scalarVariable():
             return logarithmicVariables.__sub__(self, other)
 
         # convert self and other to the SI unit system
-        selfUnit = deepcopy(self.unit)
-        otherUnit = deepcopy(other.unit)
+        selfUnit = copy(self.unit)
+        otherUnit = copy(other.unit)
  
         if scaleToSI:
             self.convert(self._unitObject._SIBaseUnit)
@@ -346,11 +363,11 @@ class scalarVariable():
 
         # determine the value and gradients
         val = self.value - other.value
-        grad = [1, -1]
-        vars = [self, other]
+
         # create the new variable
         var = variable(val, outputUnit)
-        var._addDependents(vars, grad)
+        var._addDependent(self, 1)
+        var._addDependent(other, -1)
         var._calculateUncertanty()
 
 
@@ -370,18 +387,16 @@ class scalarVariable():
         return - self + other
 
     def __mul__(self, other):
-
+    
         if not isinstance(other, scalarVariable):
             return self * variable(other)
 
         val = self.value * other.value
         outputUnit = self._unitObject * other._unitObject
 
-        grad = [other._value, self._value]
-        vars = [self, other]
-
         var = variable(val, outputUnit)
-        var._addDependents(vars, grad)
+        var._addDependent(self, other.value)
+        var._addDependent(other, self.value)
         var._calculateUncertanty()
 
         if var._unitObject._SIBaseUnit == '1' and var._unitObject != '1':
@@ -399,13 +414,13 @@ class scalarVariable():
         if str(other.unit) != '1':
             raise ValueError('The exponent can not have a unit')
 
-        selfUnit = deepcopy(self.unit)
+        selfUnit = copy(self.unit)
         outputUnit, hasToBeScaledToSI = self._unitObject ** other.value
 
         if hasToBeScaledToSI:
             self.convert(self._unitObject._SIBaseUnit)
         
-        val = self._value ** other.value
+        val = self.value ** other.value
 
         def gradSelf(valSelf, valOTher):
             if valSelf != 0:
@@ -417,13 +432,12 @@ class scalarVariable():
                 return valSelf ** valOther * np.log(valSelf)
             return 0
         
-        gradOther = np.vectorize(gradOther, otypes=[float])(self._value, other._value, other._uncert)
-        gradSelf = np.vectorize(gradSelf, otypes=[float])(self._value, other._value)
-        grad = [gradSelf, gradOther]
-        vars = [self, other]
-                
+        gradOther = np.vectorize(gradOther, otypes=[float])(self.value, other.value, other._uncert)
+        gradSelf = np.vectorize(gradSelf, otypes=[float])(self.value, other.value)
+        
         var = variable(val, outputUnit)
-        var._addDependents(vars, grad)
+        var._addDependent(self, gradSelf)
+        var._addDependent(other, gradOther)
         var._calculateUncertanty()
         
         if hasToBeScaledToSI:
@@ -438,14 +452,12 @@ class scalarVariable():
         if not isinstance(other, scalarVariable):
             return self / variable(other)
 
-        val = self._value / other._value
+        val = self.value / other.value
         outputUnit = self._unitObject / other._unitObject
 
-        grad = [1 / other._value, -self._value / (other._value**2)]
-        vars = [self, other]
-        
         var = variable(val, outputUnit)
-        var._addDependents(vars, grad)
+        var._addDependent(self, 1 / other.value)
+        var._addDependent(other, -self.value / (other.value**2))
         var._calculateUncertanty()
 
         if var._unitObject._SIBaseUnit == '1' and var._unitObject != '1':
@@ -457,14 +469,12 @@ class scalarVariable():
         if not isinstance(other, scalarVariable):
             return variable(other) / self
 
-        val = other._value / self._value
+        val = other.value / self.value
         outputUnit = other._unitObject / self._unitObject
-
-        grad = [-other._value / (self._value**2), 1 / (self._value)]
-        vars = [self, other]
-
+        
         var = variable(val, outputUnit)
-        var._addDependents(vars, grad)
+        var._addDependent(self, -other.value / (self.value**2))
+        var._addDependent(other, 1 / (self.value))
         var._calculateUncertanty()
 
         if var._unitObject._SIBaseUnit == '1' and var._unitObject != '1':
@@ -481,11 +491,8 @@ class scalarVariable():
 
         val = np.log(self.value)
 
-        vars = [self]
-        grad = [1 / self.value]
-
         var = variable(val, '1')
-        var._addDependents(vars, grad)
+        var._addDependent(self, 1 / self.value)
         var._calculateUncertanty()
 
         return var
@@ -495,11 +502,8 @@ class scalarVariable():
             raise ValueError('You can only take the base 10 log of a variable if it has no unit')
         val = np.log10(self.value)
 
-        vars = [self]
-        grad = [1 / (self.value * np.log10(self.value))]
-
         var = variable(val, '1')
-        var._addDependents(vars, grad)
+        var._addDependent(self, 1 / (self.value * np.log10(self.value)))
         var._calculateUncertanty()
 
         return var
@@ -517,15 +521,13 @@ class scalarVariable():
         outputUnit = '1'
         if self._unitObject._assertEqual('rad'):
             val = np.sin(self.value)
-            grad = [np.cos(self.value)]
+            grad = np.cos(self.value)
         else:
             val = np.sin(np.pi / 180 * self.value)
-            grad = [np.pi / 180 * np.cos(np.pi / 180 * self.value)]
+            grad = np.pi / 180 * np.cos(np.pi / 180 * self.value)
         
-        vars = [self]
-
         var = variable(val, outputUnit)
-        var._addDependents(vars, grad)
+        var._addDependent(self, grad)
         var._calculateUncertanty()
 
         return var
@@ -537,15 +539,13 @@ class scalarVariable():
         outputUnit = '1'
         if self.unit == 'rad':
             val = np.cos(self.value)
-            grad = [-np.sin(self.value)]
+            grad = -np.sin(self.value)
         else:
             val = np.cos(np.pi / 180 * self.value)
-            grad = [-np.pi / 180 * np.sin(np.pi / 180 * self.value)]
-
-        vars = [self]
-
+            grad = -np.pi / 180 * np.sin(np.pi / 180 * self.value)
+            
         var = variable(val, outputUnit)
-        var._addDependents(vars, grad)
+        var._addDependent(self, grad)
         var._calculateUncertanty()
 
         return var
@@ -557,15 +557,13 @@ class scalarVariable():
         outputUnit = '1'
         if self.unit == 'rad':
             val = np.tan(self.value)
-            grad = [2 / (np.cos(2 * self.value) + 1)]
+            grad = 2 / (np.cos(2 * self.value) + 1)
         else:
             val = np.tan(np.pi / 180 * self.value)
-            grad = [np.pi / (90 * (np.cos(np.pi / 90 * self.value) + 1))]
-
-        vars = [self]
+            grad = np.pi / (90 * (np.cos(np.pi / 90 * self.value) + 1))
 
         var = variable(val, outputUnit)
-        var._addDependents(vars, grad)
+        var._addDependent(self, grad)
         var._calculateUncertanty()
 
         return var
@@ -585,8 +583,10 @@ class scalarVariable():
     
     def max(self):
         return self
+    
     def min(self):
         return self
+    
     def mean(self):
         return self
 
@@ -597,8 +597,8 @@ class scalarVariable():
         if not self._unitObject._SIBaseUnit == other._unitObject._SIBaseUnit:
             raise ValueError(f'You cannot compare {self} and {other} as they do not have the same SI base unit')
         
-        selfUnit = deepcopy(self.unit)
-        otherUnit = deepcopy(other.unit)
+        selfUnit = copy(self.unit)
+        otherUnit = copy(other.unit)
         
         self.convert(self._unitObject._SIBaseUnit)
         other.convert(self._unitObject._SIBaseUnit)
@@ -615,8 +615,8 @@ class scalarVariable():
         if not self._unitObject._SIBaseUnit == other._unitObject._SIBaseUnit:
             raise ValueError(f'You cannot compare {self} and {other} as they do not have the same SI base unit')
         
-        selfUnit = deepcopy(self.unit)
-        otherUnit = deepcopy(other.unit)
+        selfUnit = copy(self.unit)
+        otherUnit = copy(other.unit)
         
         self.convert(self._unitObject._SIBaseUnit)
         other.convert(self._unitObject._SIBaseUnit)
@@ -634,8 +634,8 @@ class scalarVariable():
         if not self._unitObject._SIBaseUnit == other._unitObject._SIBaseUnit:
             raise ValueError(f'You cannot compare {self} and {other} as they do not have the same SI base unit')
         
-        selfUnit = deepcopy(self.unit)
-        otherUnit = deepcopy(other.unit)
+        selfUnit = copy(self.unit)
+        otherUnit = copy(other.unit)
         
         self.convert(self._unitObject._SIBaseUnit)
         other.convert(self._unitObject._SIBaseUnit)
@@ -653,8 +653,8 @@ class scalarVariable():
         if not self._unitObject._SIBaseUnit == other._unitObject._SIBaseUnit:
             raise ValueError(f'You cannot compare {self} and {other} as they do not have the same SI base unit')
         
-        selfUnit = deepcopy(self.unit)
-        otherUnit = deepcopy(other.unit)
+        selfUnit = copy(self.unit)
+        otherUnit = copy(other.unit)
         
         self.convert(self._unitObject._SIBaseUnit)
         other.convert(self._unitObject._SIBaseUnit)
@@ -672,8 +672,8 @@ class scalarVariable():
         if not self._unitObject._SIBaseUnit == other._unitObject._SIBaseUnit:
             raise ValueError(f'You cannot compare {self} and {other} as they do not have the same SI base unit')
         
-        selfUnit = deepcopy(self.unit)
-        otherUnit = deepcopy(other.unit)
+        selfUnit = copy(self.unit)
+        otherUnit = copy(other.unit)
         
         self.convert(self._unitObject._SIBaseUnit)
         other.convert(self._unitObject._SIBaseUnit)
@@ -694,8 +694,8 @@ class scalarVariable():
         if not self._unitObject._SIBaseUnit == other._unitObject._SIBaseUnit:
             raise ValueError(f'You cannot compare {self} and {other} as they do not have the same SI base unit')
         
-        selfUnit = deepcopy(self.unit)
-        otherUnit = deepcopy(other.unit)
+        selfUnit = copy(self.unit)
+        otherUnit = copy(other.unit)
         
         self.convert(self._unitObject._SIBaseUnit)
         other.convert(self._unitObject._SIBaseUnit)
@@ -712,44 +712,120 @@ class scalarVariable():
     def __hash__(self):
         return id(self)
 
-   
+
 class arrayVariable(scalarVariable):
-   
+    
+    def __init__(self, value = None, unitStr = None, uncert= None, nDigits= None, scalarVariables = None) -> None:
+        
+        if not (value is None) and not (scalarVariables is None):
+            raise ValueError('You cannot supply both values and scalarVariables')
+        
+        if not value is None:
+            self.nDigits = nDigits
+            
+            # create a unit object
+            self._unitObject = unitStr if isinstance(unitStr, unit) else unit(unitStr)        
+            
+            self.scalarVariables = []
+            if not value is None:
+                for v, u in zip(value, uncert):
+                    self.scalarVariables.append(scalarVariable(v, self._unitObject, u, nDigits))
+        else:
+            self.scalarVariables = scalarVariables
+            self._unitObject = self.scalarVariables[0]._unitObject
+            self.nDigits = self.scalarVariables[0].nDigits
+            
+    def _calculateUncertanty(self):
+        for elem in self.scalarVariables:
+            elem._calculateUncertanty()
+    
+    def addCovariance(self, var, grad: np.ndarray, unitStr: str):
+        for elem, varElem, gradElem in zip(self.scalarVariables, var, grad):
+            elem.addCovariance(varElem, gradElem, unitStr)
+    
+    def _addDependent(self, var, grad):
+        isArrayVariable = isinstance(var, arrayVariable)
+        isArrayGradient = isinstance(grad, list) or isinstance(grad, np.ndarray)
+        
+        for i, elem in enumerate(self.scalarVariables):
+            v = var[i] if isArrayVariable else var
+            g = grad[i] if isArrayGradient else grad
+            elem._addDependent(v, g)
+                        
     def __len__(self):
-        return len(self._value)
+        return len(self.scalarVariables)
 
     def __getitem__(self, index):
-        if isinstance(index, int) or isinstance(index, slice):
-            if index >= len(self):
-                raise IndexError('Index out of bounds')
-            if len(self) == 1:
-                if index != 0:
-                    raise IndexError('Index out of bound')
-                return variable(self.value, self._unitObject, self.uncert)
-            return variable(self.value[index], self._unitObject, self.uncert[index])
+        
+        if isinstance(index, int):
+            return self.scalarVariables[index]
+        elif isinstance(index, slice):
+            return arrayVariable(scalarVariables = self.scalarVariables[index])
+        if isinstance(index, list):
+            return arrayVariable(scalarVariables = [self.scalarVariables[elem] for elem in index])
         else:
-            val = [self.value[elem]for elem in index]
-            unc = [self.uncert[elem]for elem in index]
-            return variable(val, self._unitObject, unc)
+            raise NotImplementedError()
+ 
+    @property
+    def value(self):
+        return np.array([elem.value for elem in self.scalarVariables])       
+ 
+    @property
+    def uncert(self):
+        return np.array([elem.uncert for elem in self.scalarVariables])       
  
     def __setitem__(self, i, elem):
         if (type(elem) != scalarVariable):
             raise ValueError(f'You can only set an element with a scalar variable')
         if (elem.unit != self.unit):
             raise ValueError(f'You can not set an element of {self} with {elem} as they do not have the same unit')
-        self._value[i] = elem.value
-        self._uncert[i] = elem.uncert
-    
+        
+        self.scalarVariables[i] = elem
+   
     def append(self, elem):
-        if not (isinstance(elem, scalarVariable)):
-            raise ValueError(f'You can only set an element with a variable')
+        
         if (elem.unit != self.unit):
             raise ValueError(f'You can not set an element of {self} with {elem} as they do not have the same unit')
-        
-        self._value = np.append(self._value, elem.value)
-        self._uncert = np.append(self._uncert, elem.uncert)
-    
+
+        if isinstance(elem, arrayVariable):
+            elemsToAppend = [e for e in elem]
+            for e in elemsToAppend:
+                self.scalarVariables.append(e)
+        else:
+            self.scalarVariables.append(elem) 
        
+    def printUncertanty(self, value, uncert):
+        # function to print number
+        if uncert == 0 or uncert is None or np.isnan(uncert):
+            return f'{value:.{self.nDigits}g}', 0
+
+        digitsUncert = -int(np.floor(np.log10(np.abs(uncert))))
+
+        if value != 0:
+            digitsValue = -int(np.floor(np.log10(np.abs(value))))
+        else:
+            digitsValue = 0
+
+        # uncertanty
+        if digitsUncert > 0:
+            uncert = f'{uncert:.{1}g}'
+        else:
+            nDecimals = len(str(int(uncert)))
+            uncert = int(np.around(uncert, -nDecimals + 1))
+
+        # value
+        if digitsValue <= digitsUncert:
+            if digitsUncert > 0:
+                value = f'{value:.{digitsUncert}f}'
+            else:
+                value = int(np.around(value, - nDecimals + 1))
+        else:
+            value = '0'
+            if digitsUncert > 0:
+                value += '.' + ''.join(['0'] * digitsUncert)
+
+        return value, uncert 
+    
     def __str__(self, pretty=None) -> str:
         unitStr = self._unitObject.__str__(pretty=pretty)
 
@@ -778,7 +854,7 @@ class arrayVariable(scalarVariable):
             valStr.append(v)
             uncStr.append(u)
 
-        if all(self._uncert == 0) or all(elem is None for elem in self._uncert):
+        if all(self.uncert == 0) or all(elem is None for elem in self.uncert):
             out = rf''
             out += rf'['
             for i, elem in enumerate(valStr):
@@ -820,7 +896,7 @@ class arrayVariable(scalarVariable):
             out = [a**other for a in self]
 
         if all([out[0].unit == elem.unit for elem in out]):
-            return variable([elem.value for elem in out], out[0].unit, [elem.uncert for elem in out])
+            return arrayVariable(scalarVariables=out)
         
         return out
      
@@ -867,9 +943,25 @@ class arrayVariable(scalarVariable):
     
     def mean(self):
         return sum(self) / len(self)
+    
+    def convert(self, newUnit):
+        converter = self._unitObject.getConverter(newUnit)
+        for elem in self.scalarVariables:
+            elem._value = converter.convert(elem._value, useOffset=not self._unitObject.isCombinationUnit())
+            elem._uncert = converter.convert(elem._uncert, useOffset=False)
+        self._unitObject.convert(newUnit)
 
+    def __iter__(self):
+        self.n = 0
+        return self
 
+    def __next__(self):
+        if self.n < len(self):
+            out = self.scalarVariables[self.n]
+            self.n += 1
+            return out
 
+        raise StopIteration
 
 def variable(value, unit = '', uncert = None, nDigits = 3):
     # store the value and the uncertaty
@@ -877,13 +969,9 @@ def variable(value, unit = '', uncert = None, nDigits = 3):
         if input is None:
             return input
         if isinstance(input, np.ndarray):
-            if len(input) == 1:
-                return input[0]
             return input
         else:
             if isinstance(input, list):
-                if len(input) == 1:
-                    return input[0]
                 return np.array(input, dtype=float)
             else:
                 return float(input)
@@ -907,7 +995,15 @@ def variable(value, unit = '', uncert = None, nDigits = 3):
                 raise ValueError('The lenght of the value has to be equal to the lenght of the uncertanty')      
     
     if isinstance(value, np.ndarray):
-        return arrayVariable(value, unit, uncert, nDigits)
+        return arrayVariable(value = value, unitStr = unit, uncert = uncert, nDigits = nDigits)
     else:
         return scalarVariable(value, unit, uncert, nDigits)
 
+
+if __name__ == "__main__":
+    A = variable(0.003, 'L/min', 0.2)
+    print(A)
+    a = variable(0.9, '', 3)
+    print(a)    
+    
+    
